package net.serice.checksum;

import java.security.*;

public class Util {

    // This function is a wrapper around Security.getProvider() that
    // checks to see if the provider is statically registered in
    // jre/lib/security/java.security.  If not, it tries to
    // dynamically load and register the provider.  Dynamic loading
    // currently only supports BouncyCastle and Cryptix which are the
    // main providers that are not installed by default.
    public static Provider getProvider(String provider) {
        // Check if provider is statically registered.
        Provider rv = Security.getProvider(provider);
        if (rv == null) {
            String root = mapProviderToRootClass(provider);
            rv = loadProviderDynamically(root);
        }
        return rv;
    }

    // This function tries to load a "provider" dynamically using
    // "root" as the root class for the provider.  If "root" is null,
    // it will attempt to look up the root class using
    // mapProviderToRootClass() below.  If the provider can be loaded,
    // it is returned; otherwise, null is returned.
    public static Provider loadProviderDynamically(String root)
    {
        Provider rv = null;
        try {
            Class c = Class.forName(root);
            rv = (Provider)c.newInstance();
            Security.addProvider(rv);
        } catch (Exception e) {
            rv = null;
        }
        return rv;
    }

    // Map known providers to their root classes.  If no mapping is
    // known, null is returned.
    public static String mapProviderToRootClass(String provider) {
        String rv = null;
        if (provider.equalsIgnoreCase("BC")) {
            rv = "org.bouncycastle.jce.provider.BouncyCastleProvider";
        } else if (provider.equalsIgnoreCase("CryptixCrypto")) {
            rv = "cryptix.jce.provider.CryptixCrypto";
        }
        return rv;
    }

}
